@@ -137,7 +137,7 @@ class WeightProofHandler:
 
             if _sample_sub_epoch(prev_ses_block.weight, ses_block.weight, weight_to_check):  # type: ignore
                 sample_n += 1
-                segments = await self.blockchain.get_sub_epoch_challenge_segments(ses_block.height)
+                segments = await self.blockchain.get_sub_epoch_challenge_segments(ses_block.header_hash)
                 if segments is None:
                     segments = await self.__create_sub_epoch_segments(ses_block, prev_ses_block, uint32(sub_epoch_n))
                     if segments is None:
@@ -145,7 +145,7 @@ class WeightProofHandler:
                             f"failed while building segments for sub epoch {sub_epoch_n}, ses height {ses_height} "
                         )
                         return None
-                    await self.blockchain.persist_sub_epoch_challenge_segments(ses_block.height, segments)
+                    await self.blockchain.persist_sub_epoch_challenge_segments(ses_block.header_hash, segments)
                 log.debug(f"sub epoch {sub_epoch_n} has {len(segments)} segments")
                 sub_epoch_segments.extend(segments)
             prev_ses_block = ses_block
@@ -218,15 +218,59 @@ class WeightProofHandler:
         assert prev_ses_sub_block.sub_epoch_summary_included is not None
         segments = await self.__create_sub_epoch_segments(ses_sub_block, prev_ses_sub_block, uint32(count))
         assert segments is not None
-        await self.blockchain.persist_sub_epoch_challenge_segments(ses_sub_block.height, segments)
+        await self.blockchain.persist_sub_epoch_challenge_segments(ses_sub_block.header_hash, segments)
         log.debug("sub_epoch_segments done")
         return
 
+    async def create_sub_epoch_segments(self):
+        log.debug("check segments in db")
+        """
+        Creates a weight proof object
+         """
+        assert self.blockchain is not None
+        peak_height = self.blockchain.get_peak_height()
+        if peak_height is None:
+            log.error("no peak yet")
+            return None
+
+        summary_heights = self.blockchain.get_ses_heights()
+        prev_ses_block = await self.blockchain.get_block_record_from_db(self.blockchain.height_to_hash(uint32(0)))
+        if prev_ses_block is None:
+            return None
+
+        ses_blocks = await self.blockchain.get_block_records_at(summary_heights)
+        if ses_blocks is None:
+            return None
+
+        for sub_epoch_n, ses_height in enumerate(summary_heights):
+            log.debug(f"check db for sub epoch {sub_epoch_n}")
+            if ses_height > peak_height:
+                break
+            ses_block = ses_blocks[sub_epoch_n]
+            if ses_block is None or ses_block.sub_epoch_summary_included is None:
+                log.error("error while building proof")
+                return None
+            await self.__create_persist_segment(prev_ses_block, ses_block, ses_height, sub_epoch_n)
+            prev_ses_block = ses_block
+            await asyncio.sleep(2)
+        log.debug("done checking segments")
+        return
+
+    async def __create_persist_segment(self, prev_ses_block, ses_block, ses_height, sub_epoch_n):
+        segments = await self.blockchain.get_sub_epoch_challenge_segments(ses_block.header_hash)
+        if segments is None:
+            segments = await self.__create_sub_epoch_segments(ses_block, prev_ses_block, uint32(sub_epoch_n))
+            if segments is None:
+                log.error(f"failed while building segments for sub epoch {sub_epoch_n}, ses height {ses_height} ")
+                return
+            await self.blockchain.persist_sub_epoch_challenge_segments(ses_block.header_hash, segments)
+
     async def __create_sub_epoch_segments(
         self, ses_block: BlockRecord, se_start: BlockRecord, sub_epoch_n: uint32
     ) -> Optional[List[SubEpochChallengeSegment]]:
         segments: List[SubEpochChallengeSegment] = []
         start_height = await self.get_prev_two_slots_height(se_start)
+
         blocks = await self.blockchain.get_block_records_in_range(
             start_height, ses_block.height + self.constants.MAX_SUB_SLOT_BLOCKS
         )
