@@ -1,17 +1,18 @@
 import asyncio
 import logging
 import time
+from datetime import datetime
 from pathlib import Path
 from typing import Callable, Dict, List, Optional, Tuple
 
-from blspy import PrivateKey
+from blspy import PrivateKey, G1Element
 
-from chia.cmds.init import check_keys
+from chia.cmds.init_funcs import check_keys
 from chia.consensus.block_rewards import calculate_base_farmer_reward
-from chia.consensus.network_type import NetworkType
 from chia.protocols.protocol_message_types import ProtocolMessageTypes
 from chia.server.outbound_message import NodeType, make_msg
 from chia.simulator.simulator_protocol import FarmNewBlockProtocol
+from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.bech32m import decode_puzzle_hash, encode_puzzle_hash
 from chia.util.byte_types import hexstr_to_bytes
@@ -21,6 +22,7 @@ from chia.util.path import path_from_root
 from chia.util.ws_message import WsRpcMessage, create_payload_dict
 from chia.wallet.cc_wallet.cc_wallet import CCWallet
 from chia.wallet.rl_wallet.rl_wallet import RLWallet
+from chia.wallet.did_wallet.did_wallet import DIDWallet
 from chia.wallet.trade_record import TradeRecord
 from chia.wallet.transaction_record import TransactionRecord
 from chia.wallet.util.backup_utils import download_backup, get_backup_info, upload_backup
@@ -70,6 +72,7 @@ class WalletRpcApi:
             "/create_backup": self.create_backup,
             "/get_transaction_count": self.get_transaction_count,
             "/get_farmed_amount": self.get_farmed_amount,
+            "/create_signed_transaction": self.create_signed_transaction,
             # Coloured coins and trading
             "/cc_set_name": self.cc_set_name,
             "/cc_get_name": self.cc_get_name,
@@ -81,6 +84,16 @@ class WalletRpcApi:
             "/get_trade": self.get_trade,
             "/get_all_trades": self.get_all_trades,
             "/cancel_trade": self.cancel_trade,
+            # DID Wallet
+            "/did_update_recovery_ids": self.did_update_recovery_ids,
+            "/did_spend": self.did_spend,
+            "/did_get_pubkey": self.did_get_pubkey,
+            "/did_get_did": self.did_get_did,
+            "/did_recovery_spend": self.did_recovery_spend,
+            "/did_get_recovery_list": self.did_get_recovery_list,
+            "/did_create_attest": self.did_create_attest,
+            "/did_get_information_needed_for_recovery": self.did_get_information_needed_for_recovery,
+            "/did_create_backup_file": self.did_create_backup_file,
             # RL wallet
             "/rl_set_user_info": self.rl_set_user_info,
             "/send_clawback_transaction:": self.send_clawback_transaction,
@@ -295,7 +308,7 @@ class WalletRpcApi:
 
         return {"wallets": wallets}
 
-    async def _create_backup_and_upload(self, host):
+    async def _create_backup_and_upload(self, host) -> None:
         assert self.service.wallet_state_manager is not None
         try:
             if "testing" in self.service.config and self.service.config["testing"] is True:
@@ -334,7 +347,7 @@ class WalletRpcApi:
                 cc_wallet = await CCWallet.create_wallet_for_cc(wallet_state_manager, main_wallet, request["colour"])
                 asyncio.create_task(self._create_backup_and_upload(host))
                 return {"type": cc_wallet.type()}
-        if request["wallet_type"] == "rl_wallet":
+        elif request["wallet_type"] == "rl_wallet":
             if request["rl_type"] == "admin":
                 log.info("Create rl admin wallet")
                 rl_admin: RLWallet = await RLWallet.create_rl_admin(wallet_state_manager)
@@ -364,6 +377,52 @@ class WalletRpcApi:
                     "type": rl_user.type(),
                     "pubkey": rl_user.rl_info.user_pubkey.hex(),
                 }
+        elif request["wallet_type"] == "did_wallet":
+            if request["did_type"] == "new":
+                backup_dids = []
+                num_needed = 0
+                for d in request["backup_dids"]:
+                    backup_dids.append(hexstr_to_bytes(d))
+                if len(backup_dids) > 0:
+                    num_needed = uint64(request["num_of_backup_ids_needed"])
+                did_wallet: DIDWallet = await DIDWallet.create_new_did_wallet(
+                    wallet_state_manager,
+                    main_wallet,
+                    int(request["amount"]),
+                    backup_dids,
+                    uint64(num_needed),
+                )
+                my_did = did_wallet.get_my_DID()
+                return {
+                    "success": True,
+                    "type": did_wallet.type(),
+                    "my_did": my_did,
+                    "wallet_id": did_wallet.id(),
+                }
+            elif request["did_type"] == "recovery":
+                did_wallet = await DIDWallet.create_new_did_wallet_from_recovery(
+                    wallet_state_manager, main_wallet, request["filename"]
+                )
+                assert did_wallet.did_info.temp_coin is not None
+                assert did_wallet.did_info.temp_puzhash is not None
+                assert did_wallet.did_info.temp_pubkey is not None
+                my_did = did_wallet.get_my_DID()
+                coin_name = did_wallet.did_info.temp_coin.name().hex()
+                coin_list = did_wallet.did_info.temp_coin.as_list()
+                newpuzhash = did_wallet.did_info.temp_puzhash
+                pubkey = did_wallet.did_info.temp_pubkey
+                return {
+                    "success": True,
+                    "type": did_wallet.type(),
+                    "my_did": my_did,
+                    "wallet_id": did_wallet.id(),
+                    "coin_name": coin_name,
+                    "coin_list": coin_list,
+                    "newpuzhash": newpuzhash.hex(),
+                    "pubkey": pubkey.hex(),
+                    "backup_dids": did_wallet.did_info.backup_ids,
+                    "num_verifications_required": did_wallet.did_info.num_of_backup_ids_needed,
+                }
 
     ##########################################################################################
     # Wallet
@@ -393,7 +452,7 @@ class WalletRpcApi:
 
     async def get_transaction(self, request: Dict) -> Dict:
         assert self.service.wallet_state_manager is not None
-        transaction_id: bytes32 = bytes32(bytes.fromhex(request["transaction_id"]))
+        transaction_id: bytes32 = bytes32(hexstr_to_bytes(request["transaction_id"]))
         tr: Optional[TransactionRecord] = await self.service.wallet_state_manager.get_transaction(transaction_id)
         if tr is None:
             raise ValueError(f"Transaction 0x{transaction_id.hex()} not found")
@@ -430,9 +489,9 @@ class WalletRpcApi:
             "wallet_id": wallet_id,
         }
 
-    async def get_initial_freeze_period(self):
-        freeze_period = self.service.constants.INITIAL_FREEZE_PERIOD
-        return {"INITIAL_FREEZE_PERIOD": freeze_period}
+    async def get_initial_freeze_period(self, _: Dict):
+        freeze_period = self.service.constants.INITIAL_FREEZE_END_TIMESTAMP
+        return {"INITIAL_FREEZE_END_TIMESTAMP": freeze_period}
 
     async def get_next_address(self, request: Dict) -> Dict:
         """
@@ -468,14 +527,9 @@ class WalletRpcApi:
         if await self.service.wallet_state_manager.synced() is False:
             raise ValueError("Wallet needs to be fully synced before sending transactions")
 
-        if (
-            self.service.wallet_state_manager.blockchain.get_peak_height()
-            < self.service.constants.INITIAL_FREEZE_PERIOD
-        ):
-            raise ValueError(f"No transactions before block height: {self.service.constants.INITIAL_FREEZE_PERIOD}")
-
-        if self.service.constants.NETWORK_TYPE is NetworkType.MAINNET:
-            raise ValueError("Sending transactions not supported, please update your client.")
+        if int(time.time()) < self.service.constants.INITIAL_FREEZE_END_TIMESTAMP:
+            end_date = datetime.fromtimestamp(float(self.service.constants.INITIAL_FREEZE_END_TIMESTAMP))
+            raise ValueError(f"No transactions before: {end_date}")
 
         wallet_id = int(request["wallet_id"])
         wallet = self.service.wallet_state_manager.wallets[wallet_id]
@@ -488,9 +542,9 @@ class WalletRpcApi:
             fee = uint64(request["fee"])
         else:
             fee = uint64(0)
-        tx: TransactionRecord = await wallet.generate_signed_transaction(amount, puzzle_hash, fee)
-
-        await wallet.push_transaction(tx)
+        async with self.service.wallet_state_manager.tx_lock:
+            tx: TransactionRecord = await wallet.generate_signed_transaction(amount, puzzle_hash, fee)
+            await wallet.push_transaction(tx)
 
         # Transaction may not have been included in the mempool yet. Use get_transaction to check.
         return {
@@ -540,9 +594,9 @@ class WalletRpcApi:
             fee = uint64(request["fee"])
         else:
             fee = uint64(0)
-
-        tx: TransactionRecord = await wallet.generate_signed_transaction([amount], [puzzle_hash], fee)
-        await wallet.wallet_state_manager.add_pending_transaction(tx)
+        async with self.service.wallet_state_manager.tx_lock:
+            tx: TransactionRecord = await wallet.generate_signed_transaction([amount], [puzzle_hash], fee)
+            await wallet.push_transaction(tx)
 
         return {
             "transaction": tx,
@@ -657,6 +711,141 @@ class WalletRpcApi:
         backup_info = get_backup_info(file_path, sk)
         return {"backup_info": backup_info}
 
+    ##########################################################################################
+    # Distributed Identities
+    ##########################################################################################
+
+    async def did_update_recovery_ids(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        recovery_list = []
+        for _ in request["new_list"]:
+            recovery_list.append(hexstr_to_bytes(_))
+        if "num_verifications_required" in request:
+            new_amount_verifications_required = uint64(request["num_verifications_required"])
+        else:
+            new_amount_verifications_required = len(recovery_list)
+        success = await wallet.update_recovery_list(recovery_list, new_amount_verifications_required)
+        # Update coin with new ID info
+        updated_puz = await wallet.get_new_puzzle()
+        spend_bundle = await wallet.create_spend(updated_puz.get_tree_hash())
+        if spend_bundle is not None and success:
+            return {"success": True}
+        return {"success": False}
+
+    async def did_spend(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        spend_bundle = await wallet.create_spend(request["puzzlehash"])
+        if spend_bundle is not None:
+            return {"success": True}
+        return {"success": False}
+
+    async def did_get_did(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        my_did: str = wallet.get_my_DID()
+        coins = await wallet.select_coins(1)
+        if coins is None or coins == set():
+            return {"success": True, "wallet_id": wallet_id, "my_did": my_did}
+        else:
+            coin = coins.pop()
+            return {"success": True, "wallet_id": wallet_id, "my_did": my_did, "coin_id": coin.name()}
+
+    async def did_get_recovery_list(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        recovery_list = wallet.did_info.backup_ids
+        recover_hex_list = []
+        for _ in recovery_list:
+            recover_hex_list.append(_.hex())
+        return {
+            "success": True,
+            "wallet_id": wallet_id,
+            "recover_list": recover_hex_list,
+            "num_required": wallet.did_info.num_of_backup_ids_needed,
+        }
+
+    async def did_recovery_spend(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        if len(request["attest_filenames"]) < wallet.did_info.num_of_backup_ids_needed:
+            return {"success": False, "reason": "insufficient messages"}
+
+        (
+            info_list,
+            message_spend_bundle,
+        ) = await wallet.load_attest_files_for_recovery_spend(request["attest_filenames"])
+
+        if "pubkey" in request:
+            pubkey = G1Element.from_bytes(hexstr_to_bytes(request["pubkey"]))
+        else:
+            assert wallet.did_info.temp_pubkey is not None
+            pubkey = wallet.did_info.temp_pubkey
+
+        if "puzhash" in request:
+            puzhash = hexstr_to_bytes(request["puzhash"])
+        else:
+            assert wallet.did_info.temp_puzhash is not None
+            puzhash = wallet.did_info.temp_puzhash
+
+        success = await wallet.recovery_spend(
+            wallet.did_info.temp_coin,
+            puzhash,
+            info_list,
+            pubkey,
+            message_spend_bundle,
+        )
+        return {"success": success}
+
+    async def did_get_pubkey(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        pubkey = bytes((await wallet.wallet_state_manager.get_unused_derivation_record(wallet_id)).pubkey).hex()
+        return {"success": True, "pubkey": pubkey}
+
+    async def did_create_attest(self, request):
+        wallet_id = int(request["wallet_id"])
+        wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        info = await wallet.get_info_for_recovery()
+        coin = hexstr_to_bytes(request["coin_name"])
+        pubkey = G1Element.from_bytes(hexstr_to_bytes(request["pubkey"]))
+        spend_bundle = await wallet.create_attestment(
+            coin, hexstr_to_bytes(request["puzhash"]), pubkey, request["filename"]
+        )
+        if spend_bundle is not None:
+            return {
+                "success": True,
+                "message_spend_bundle": bytes(spend_bundle).hex(),
+                "info": [info[0].hex(), info[1].hex(), info[2]],
+            }
+        else:
+            return {"success": False}
+
+    async def did_get_information_needed_for_recovery(self, request):
+        wallet_id = int(request["wallet_id"])
+        did_wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+        my_did = did_wallet.get_my_DID()
+        coin_name = did_wallet.did_info.temp_coin.name().hex()
+        return {
+            "success": True,
+            "wallet_id": wallet_id,
+            "my_did": my_did,
+            "coin_name": coin_name,
+            "newpuzhash": did_wallet.did_info.temp_puzhash,
+            "pubkey": did_wallet.did_info.temp_pubkey,
+            "backup_dids": did_wallet.did_info.backup_ids,
+        }
+
+    async def did_create_backup_file(self, request):
+        try:
+            wallet_id = int(request["wallet_id"])
+            did_wallet: DIDWallet = self.service.wallet_state_manager.wallets[wallet_id]
+            did_wallet.create_backup(request["filename"])
+            return {"wallet_id": wallet_id, "success": True}
+        except Exception:
+            return {"wallet_id": wallet_id, "success": False}
+
     ##########################################################################################
     # Rate Limited Wallet
     ##########################################################################################
@@ -684,8 +873,9 @@ class WalletRpcApi:
         wallet: RLWallet = self.service.wallet_state_manager.wallets[wallet_id]
 
         fee = int(request["fee"])
-        tx = await wallet.clawback_rl_coin_transaction(fee)
-        await wallet.push_transaction(tx)
+        async with self.service.wallet_state_manager.tx_lock:
+            tx = await wallet.clawback_rl_coin_transaction(fee)
+            await wallet.push_transaction(tx)
 
         # Transaction may not have been included in the mempool yet. Use get_transaction to check.
         return {
@@ -728,3 +918,37 @@ class WalletRpcApi:
             "fee_amount": fee_amount,
             "last_height_farmed": last_height_farmed,
         }
+
+    async def create_signed_transaction(self, request):
+        if "additions" not in request or len(request["additions"]) < 1:
+            raise ValueError("Specify additions list")
+
+        additions: List[Dict] = request["additions"]
+        amount_0: uint64 = uint64(additions[0]["amount"])
+        assert amount_0 <= self.service.constants.MAX_COIN_AMOUNT
+        puzzle_hash_0 = hexstr_to_bytes(additions[0]["puzzle_hash"])
+        if len(puzzle_hash_0) != 32:
+            raise ValueError(f"Address must be 32 bytes. {puzzle_hash_0}")
+
+        additional_outputs = []
+        for addition in additions[1:]:
+            receiver_ph = hexstr_to_bytes(addition["puzzle_hash"])
+            if len(receiver_ph) != 32:
+                raise ValueError(f"Address must be 32 bytes. {receiver_ph}")
+            amount = uint64(addition["amount"])
+            if amount > self.service.constants.MAX_COIN_AMOUNT:
+                raise ValueError(f"Coin amount cannot exceed {self.service.constants.MAX_COIN_AMOUNT}")
+            additional_outputs.append({"puzzlehash": receiver_ph, "amount": amount})
+
+        fee = uint64(0)
+        if "fee" in request:
+            fee = uint64(request["fee"])
+
+        coins = None
+        if "coins" in request and len(request["coins"]) > 0:
+            coins = set([Coin.from_json_dict(coin_json) for coin_json in request["coins"]])
+
+        signed_tx = await self.service.wallet_state_manager.main_wallet.generate_signed_transaction(
+            amount_0, puzzle_hash_0, fee, coins=coins, ignore_max_send_amount=True, primaries=additional_outputs
+        )
+        return {"signed_tx": signed_tx}
