@@ -5,7 +5,7 @@ import logging
 import random
 import time
 import traceback
-from typing import Callable, Dict, List, Optional, Tuple
+from typing import Callable, Dict, List, Optional, Tuple, Set
 
 from chiavdf import create_discriminant
 
@@ -30,7 +30,7 @@ from chia.types.blockchain_format.slots import (
 from chia.types.blockchain_format.sub_epoch_summary import SubEpochSummary
 from chia.types.blockchain_format.vdf import VDFInfo, VDFProof
 from chia.types.end_of_slot_bundle import EndOfSubSlotBundle
-from chia.util.ints import uint8, uint32, uint64
+from chia.util.ints import uint8, uint32, uint64, uint128
 
 log = logging.getLogger(__name__)
 
@@ -60,32 +60,37 @@ class Timelord:
         # Last end of subslot bundle, None if we built a peak on top of it.
         self.new_subslot_end: Optional[EndOfSubSlotBundle] = None
         # Last state received. Can either be a new peak or a new EndOfSubslotBundle.
-        self.last_state: Optional[LastState] = None
         # Unfinished block info, iters adjusted to the last peak.
-        self.unfinished_blocks: List[timelord_protocol.NewUnfinishedBlock] = []
+        self.unfinished_blocks: List[timelord_protocol.NewUnfinishedBlockTimelord] = []
         # Signage points iters, adjusted to the last peak.
         self.signage_point_iters: List[Tuple[uint64, uint8]] = []
         # For each chain, send those info when the process spawns.
         self.iters_to_submit: Dict[Chain, List[uint64]] = {}
         self.iters_submitted: Dict[Chain, List[uint64]] = {}
+        self.iters_finished: Set = set()
         # For each iteration submitted, know if it's a signage point, an infusion point or an end of slot.
         self.iteration_to_proof_type: Dict[uint64, IterationType] = {}
         # List of proofs finished.
-        self.proofs_finished: List[Tuple[Chain, VDFInfo, VDFProof]] = []
+        self.proofs_finished: List[Tuple[Chain, VDFInfo, VDFProof, int]] = []
         # Data to send at vdf_client initialization.
-        self.overflow_blocks: List[timelord_protocol.NewUnfinishedBlock] = []
+        self.overflow_blocks: List[timelord_protocol.NewUnfinishedBlockTimelord] = []
+        # Incremented each time `reset_chains` has been called.
+        # Used to label proofs in `finished_proofs` and to only filter proofs corresponding to the most recent state.
+        self.num_resets: int = 0
 
         self.process_communication_tasks: List[asyncio.Task] = []
         self.main_loop = None
         self.vdf_server = None
         self._shut_down = False
-        self.vdf_failures: List[Chain] = []
+        self.vdf_failures: List[Tuple[Chain, Optional[int]]] = []
         self.vdf_failures_count: int = 0
+        self.vdf_failure_time: float = 0
         self.total_unfinished: int = 0
         self.total_infused: int = 0
         self.state_changed_callback: Optional[Callable] = None
         self.sanitizer_mode = self.config["sanitizer_mode"]
         self.pending_bluebox_info: List[timelord_protocol.RequestCompactProofOfTime] = []
+        self.last_active_time = time.time()
 
     async def _start(self):
         self.lock: asyncio.Lock = asyncio.Lock()
@@ -117,10 +122,10 @@ class Timelord:
     async def _handle_client(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter):
         async with self.lock:
             client_ip = writer.get_extra_info("peername")[0]
-            log.info(f"New timelord connection from client: {client_ip}.")
+            log.debug(f"New timelord connection from client: {client_ip}.")
             if client_ip in self.ip_whitelist:
                 self.free_clients.append((client_ip, reader, writer))
-                log.info(f"Added new VDF client {client_ip}.")
+                log.debug(f"Added new VDF client {client_ip}.")
                 for ip, end_time in list(self.potential_free_clients):
                     if ip == client_ip:
                         self.potential_free_clients.remove((ip, end_time))
@@ -149,7 +154,7 @@ class Timelord:
         except ConnectionResetError as e:
             log.error(f"{e}")
 
-    def _can_infuse_unfinished_block(self, block: timelord_protocol.NewUnfinishedBlock) -> Optional[uint64]:
+    def _can_infuse_unfinished_block(self, block: timelord_protocol.NewUnfinishedBlockTimelord) -> Optional[uint64]:
         assert self.last_state is not None
         sub_slot_iters = self.last_state.get_sub_slot_iters()
         difficulty = self.last_state.get_difficulty()
@@ -186,18 +191,21 @@ class Timelord:
                 )
                 return None
             if self.last_state.reward_challenge_cache[found_index][1] > block_sp_total_iters:
-                log.error(
-                    f"Will not infuse unfinished block {block.rc_prev}, sp total iters: {block_sp_total_iters}, "
-                    f"because its iters are too low"
-                )
+                if not is_overflow_block(self.constants, block.reward_chain_block.signage_point_index):
+                    log.error(
+                        f"Will not infuse unfinished block {block.rc_prev}, sp total iters: {block_sp_total_iters}, "
+                        f"because its iters are too low"
+                    )
                 return None
 
         if new_block_iters > 0:
             return new_block_iters
         return None
 
-    async def _reset_chains(self, first_run=False):
+    async def _reset_chains(self, first_run=False, only_eos=False):
         # First, stop all chains.
+        self.last_active_time = time.time()
+        log.debug("Resetting chains")
         ip_iters = self.last_state.get_last_ip()
         sub_slot_iters = self.last_state.get_sub_slot_iters()
 
@@ -217,24 +225,30 @@ class Timelord:
             assert k * iters_per_signage < sub_slot_iters
         # Adjust all unfinished blocks iterations to the peak.
         new_unfinished_blocks = []
+        self.iters_finished = set()
         self.proofs_finished = []
+        self.num_resets += 1
         for chain in [Chain.CHALLENGE_CHAIN, Chain.REWARD_CHAIN, Chain.INFUSED_CHALLENGE_CHAIN]:
             self.iters_to_submit[chain] = []
             self.iters_submitted[chain] = []
         self.iteration_to_proof_type = {}
-        for block in self.unfinished_blocks:
-            new_block_iters: Optional[uint64] = self._can_infuse_unfinished_block(block)
-            if new_block_iters:
-                new_unfinished_blocks.append(block)
-                for chain in [Chain.REWARD_CHAIN, Chain.CHALLENGE_CHAIN]:
-                    self.iters_to_submit[chain].append(new_block_iters)
-                if self.last_state.get_deficit() < self.constants.MIN_BLOCKS_PER_CHALLENGE_BLOCK:
-                    self.iters_to_submit[Chain.INFUSED_CHALLENGE_CHAIN].append(new_block_iters)
-                self.iteration_to_proof_type[new_block_iters] = IterationType.INFUSION_POINT
+        if not only_eos:
+            for block in self.unfinished_blocks + self.overflow_blocks:
+                new_block_iters: Optional[uint64] = self._can_infuse_unfinished_block(block)
+                # Does not add duplicates, or blocks that we cannot infuse
+                if new_block_iters and new_block_iters not in self.iters_to_submit[Chain.CHALLENGE_CHAIN]:
+                    if block not in self.unfinished_blocks:
+                        self.total_unfinished += 1
+                    new_unfinished_blocks.append(block)
+                    for chain in [Chain.REWARD_CHAIN, Chain.CHALLENGE_CHAIN]:
+                        self.iters_to_submit[chain].append(new_block_iters)
+                    if self.last_state.get_deficit() < self.constants.MIN_BLOCKS_PER_CHALLENGE_BLOCK:
+                        self.iters_to_submit[Chain.INFUSED_CHALLENGE_CHAIN].append(new_block_iters)
+                    self.iteration_to_proof_type[new_block_iters] = IterationType.INFUSION_POINT
         # Remove all unfinished blocks that have already passed.
         self.unfinished_blocks = new_unfinished_blocks
         # Signage points.
-        if len(self.signage_point_iters) > 0:
+        if not only_eos and len(self.signage_point_iters) > 0:
             count_signage = 0
             for signage, k in self.signage_point_iters:
                 for chain in [Chain.CHALLENGE_CHAIN, Chain.REWARD_CHAIN]:
@@ -257,12 +271,43 @@ class Timelord:
                 assert iteration > 0
 
     async def _handle_new_peak(self):
+        assert self.new_peak is not None
         self.last_state.set_state(self.new_peak)
+
+        if self.total_unfinished > 0:
+            remove_unfinished = []
+            for unf_block_timelord in self.unfinished_blocks + self.overflow_blocks:
+                if (
+                    unf_block_timelord.reward_chain_block.get_hash()
+                    == self.new_peak.reward_chain_block.get_unfinished().get_hash()
+                ):
+                    if unf_block_timelord not in self.unfinished_blocks:
+                        # We never got the EOS for this, but we have the block in overflow list
+                        self.total_unfinished += 1
+
+                    remove_unfinished.append(unf_block_timelord)
+            if len(remove_unfinished) > 0:
+                self.total_infused += 1
+            for block in remove_unfinished:
+                if block in self.unfinished_blocks:
+                    self.unfinished_blocks.remove(block)
+                if block in self.overflow_blocks:
+                    self.overflow_blocks.remove(block)
+            infusion_rate = round(self.total_infused / self.total_unfinished * 100.0, 2)
+            log.info(
+                f"Total unfinished blocks: {self.total_unfinished}. "
+                f"Total infused blocks: {self.total_infused}. "
+                f"Infusion rate: {infusion_rate}%."
+            )
+
         self.new_peak = None
         await self._reset_chains()
 
     async def _handle_subslot_end(self):
         self.last_state.set_state(self.new_subslot_end)
+        for block in self.unfinished_blocks:
+            if self._can_infuse_unfinished_block(block) is not None:
+                self.total_unfinished += 1
         self.new_subslot_end = None
         await self._reset_chains()
 
@@ -287,21 +332,23 @@ class Timelord:
                 self.unspawned_chains = self.unspawned_chains[1:]
                 self.chain_start_time[picked_chain] = time.time()
 
-            log.info(f"Mapping free vdf_client with chain: {picked_chain}.")
+            log.debug(f"Mapping free vdf_client with chain: {picked_chain}.")
             self.process_communication_tasks.append(
                 asyncio.create_task(
-                    self._do_process_communication(picked_chain, challenge, initial_form, ip, reader, writer)
+                    self._do_process_communication(
+                        picked_chain, challenge, initial_form, ip, reader, writer, proof_label=self.num_resets
+                    )
                 )
             )
 
     async def _submit_iterations(self):
         for chain in [Chain.CHALLENGE_CHAIN, Chain.REWARD_CHAIN, Chain.INFUSED_CHALLENGE_CHAIN]:
             if chain in self.allows_iters:
-                # log.info(f"Trying to submit for chain {chain}.")
                 _, _, writer = self.chain_type_to_stream[chain]
                 for iteration in self.iters_to_submit[chain]:
                     if iteration in self.iters_submitted[chain]:
                         continue
+                    log.debug(f"Submitting iterations to {chain}: {iteration}")
                     assert iteration > 0
                     prefix = str(len(str(iteration)))
                     if len(str(iteration)) < 10:
@@ -313,10 +360,12 @@ class Timelord:
 
     def _clear_proof_list(self, iters: uint64):
         return [
-            (chain, info, proof) for chain, info, proof in self.proofs_finished if info.number_of_iterations != iters
+            (chain, info, proof, label)
+            for chain, info, proof, label in self.proofs_finished
+            if info.number_of_iterations != iters
         ]
 
-    async def _check_for_new_sp(self):
+    async def _check_for_new_sp(self, iter_to_look_for: uint64):
         signage_iters = [
             iteration for iteration, t in self.iteration_to_proof_type.items() if t == IterationType.SIGNAGE_POINT
         ]
@@ -324,13 +373,13 @@ class Timelord:
             return
         to_remove = []
         for potential_sp_iters, signage_point_index in self.signage_point_iters:
-            if potential_sp_iters not in signage_iters:
+            if potential_sp_iters not in signage_iters or potential_sp_iters != iter_to_look_for:
                 continue
             signage_iter = potential_sp_iters
             proofs_with_iter = [
                 (chain, info, proof)
-                for chain, info, proof in self.proofs_finished
-                if info.number_of_iterations == signage_iter
+                for chain, info, proof, label in self.proofs_finished
+                if info.number_of_iterations == signage_iter and label == self.num_resets
             ]
             # Wait for both cc and rc to have the signage point.
             if len(proofs_with_iter) == 2:
@@ -348,12 +397,13 @@ class Timelord:
                 if cc_info is None or cc_proof is None or rc_info is None or rc_proof is None:
                     log.error(f"Insufficient signage point data {signage_iter}")
                     continue
+                self.iters_finished.add(iter_to_look_for)
+                self.last_active_time = time.time()
 
-                if rc_info.challenge != self.last_state.get_challenge(Chain.REWARD_CHAIN):
-                    log.warning(
-                        f"SP: Do not have correct challenge {self.last_state.get_challenge(Chain.REWARD_CHAIN).hex()}"
-                        f" has {rc_info.challenge}"
-                    )
+                rc_challenge = self.last_state.get_challenge(Chain.REWARD_CHAIN)
+                if rc_info.challenge != rc_challenge:
+                    assert rc_challenge is not None
+                    log.warning(f"SP: Do not have correct challenge {rc_challenge.hex()}" f" has {rc_info.challenge}")
                     # This proof is on an outdated challenge, so don't use it
                     continue
                 iters_from_sub_slot_start = cc_info.number_of_iterations + self.last_state.get_last_ip()
@@ -387,15 +437,19 @@ class Timelord:
         for r in to_remove:
             self.signage_point_iters.remove(r)
 
-    async def _check_for_new_ip(self):
+    async def _check_for_new_ip(self, iter_to_look_for: uint64):
+        if len(self.unfinished_blocks) == 0:
+            return
         infusion_iters = [
             iteration for iteration, t in self.iteration_to_proof_type.items() if t == IterationType.INFUSION_POINT
         ]
         for iteration in infusion_iters:
+            if iteration != iter_to_look_for:
+                continue
             proofs_with_iter = [
                 (chain, info, proof)
-                for chain, info, proof in self.proofs_finished
-                if info.number_of_iterations == iteration
+                for chain, info, proof, label in self.proofs_finished
+                if info.number_of_iterations == iteration and label == self.num_resets
             ]
             if self.last_state.get_challenge(Chain.INFUSED_CHALLENGE_CHAIN) is not None:
                 chain_count = 3
@@ -412,11 +466,13 @@ class Timelord:
                             self.last_state.get_sub_slot_iters(),
                             self.last_state.get_difficulty(),
                         )
-                    except Exception:
+                    except Exception as e:
+                        log.error(f"Error {e}")
                         continue
                     if ip_iters - self.last_state.get_last_ip() == iteration:
                         block = unfinished_block
                         break
+                assert ip_iters is not None
                 if block is not None:
                     ip_total_iters = self.last_state.get_total_iters() + iteration
                     challenge = block.reward_chain_block.get_hash()
@@ -440,17 +496,19 @@ class Timelord:
                         log.error(f"Insufficient VDF proofs for infusion point ch: {challenge} iterations:{iteration}")
                         return
 
-                    if rc_info.challenge != self.last_state.get_challenge(Chain.REWARD_CHAIN):
+                    rc_challenge = self.last_state.get_challenge(Chain.REWARD_CHAIN)
+                    if rc_info.challenge != rc_challenge:
+                        assert rc_challenge is not None
                         log.warning(
-                            f"Do not have correct challenge {self.last_state.get_challenge(Chain.REWARD_CHAIN).hex()} "
+                            f"Do not have correct challenge {rc_challenge.hex()} "
                             f"has {rc_info.challenge}, partial hash {block.reward_chain_block.get_hash()}"
                         )
                         # This proof is on an outdated challenge, so don't use it
                         continue
 
-                    self.unfinished_blocks.remove(block)
-                    self.total_infused += 1
-                    log.info(f"Generated infusion point for challenge: {challenge} iterations: {iteration}.")
+                    self.iters_finished.add(iter_to_look_for)
+                    self.last_active_time = time.time()
+                    log.debug(f"Generated infusion point for challenge: {challenge} iterations: {iteration}.")
 
                     overflow = is_overflow_block(self.constants, block.reward_chain_block.signage_point_index)
 
@@ -495,8 +553,10 @@ class Timelord:
                         is_transaction_block = True
                         height: uint32 = uint32(0)
                     else:
-                        is_transaction_block = self.last_state.get_last_block_total_iters() < sp_total_iters
-                        height: uint32 = self.last_state.get_height() + 1
+                        last_block_ti = self.last_state.get_last_block_total_iters()
+                        assert last_block_ti is not None
+                        is_transaction_block = last_block_ti < sp_total_iters
+                        height = uint32(self.last_state.get_height() + 1)
 
                     if height < 5:
                         # Don't directly update our state for the first few blocks, because we cannot validate
@@ -504,9 +564,9 @@ class Timelord:
                         return
 
                     new_reward_chain_block = RewardChainBlock(
-                        self.last_state.get_weight() + block.difficulty,
+                        uint128(self.last_state.get_weight() + block.difficulty),
                         height,
-                        ip_total_iters,
+                        uint128(ip_total_iters),
                         block.reward_chain_block.signage_point_index,
                         block.reward_chain_block.pos_ss_cc_challenge_hash,
                         block.reward_chain_block.proof_of_space,
@@ -561,34 +621,30 @@ class Timelord:
                     self.new_peak = timelord_protocol.NewPeakTimelord(
                         new_reward_chain_block,
                         block.difficulty,
-                        new_deficit,
+                        uint8(new_deficit),
                         block.sub_slot_iters,
                         new_sub_epoch_summary,
                         self.last_state.reward_challenge_cache,
-                        last_csb_or_eos,
+                        uint128(last_csb_or_eos),
                         passed_ses_height_but_not_yet_included,
                     )
-                    if self.total_unfinished > 0:
-                        infusion_rate = int(self.total_infused / self.total_unfinished * 100)
-                        log.info(
-                            f"Total unfinished blocks: {self.total_unfinished}."
-                            f"Total infused blocks: {self.total_infused}."
-                            f"Infusion rate: {infusion_rate}."
-                        )
+
                     await self._handle_new_peak()
                     # Break so we alternate between checking SP and IP
                     break
 
-    async def _check_for_end_of_subslot(self):
+    async def _check_for_end_of_subslot(self, iter_to_look_for: uint64):
         left_subslot_iters = [
             iteration for iteration, t in self.iteration_to_proof_type.items() if t == IterationType.END_OF_SUBSLOT
         ]
         if len(left_subslot_iters) == 0:
             return
+        if left_subslot_iters[0] != iter_to_look_for:
+            return
         chains_finished = [
             (chain, info, proof)
-            for chain, info, proof in self.proofs_finished
-            if info.number_of_iterations == left_subslot_iters[0]
+            for chain, info, proof, label in self.proofs_finished
+            if info.number_of_iterations == left_subslot_iters[0] and label == self.num_resets
         ]
         if self.last_state.get_challenge(Chain.INFUSED_CHALLENGE_CHAIN) is not None:
             chain_count = 3
@@ -613,14 +669,15 @@ class Timelord:
                     icc_ip_proof = proof
             assert cc_proof is not None and rc_proof is not None and cc_vdf is not None and rc_vdf is not None
 
-            if rc_vdf.challenge != self.last_state.get_challenge(Chain.REWARD_CHAIN):
-                log.warning(
-                    f"Do not have correct challenge {self.last_state.get_challenge(Chain.REWARD_CHAIN).hex()} has"
-                    f" {rc_vdf.challenge}"
-                )
+            rc_challenge = self.last_state.get_challenge(Chain.REWARD_CHAIN)
+            if rc_vdf.challenge != rc_challenge:
+                assert rc_challenge is not None
+                log.warning(f"Do not have correct challenge {rc_challenge.hex()} has" f" {rc_vdf.challenge}")
                 # This proof is on an outdated challenge, so don't use it
                 return
-            log.info("Collected end of subslot vdfs.")
+            log.debug("Collected end of subslot vdfs.")
+            self.iters_finished.add(iter_to_look_for)
+            self.last_active_time = time.time()
             iters_from_sub_slot_start = cc_vdf.number_of_iterations + self.last_state.get_last_ip()
             cc_vdf = dataclasses.replace(cc_vdf, number_of_iterations=iters_from_sub_slot_start)
             if icc_ip_vdf is not None:
@@ -643,7 +700,11 @@ class Timelord:
             icc_sub_slot: Optional[InfusedChallengeChainSubSlot] = (
                 None if icc_ip_vdf is None else InfusedChallengeChainSubSlot(icc_ip_vdf)
             )
-            icc_sub_slot_hash = icc_sub_slot.get_hash() if self.last_state.get_deficit() == 0 else None
+            if self.last_state.get_deficit() == 0:
+                assert icc_sub_slot is not None
+                icc_sub_slot_hash = icc_sub_slot.get_hash()
+            else:
+                icc_sub_slot_hash = None
             next_ses: Optional[SubEpochSummary] = self.last_state.get_next_sub_epoch_summary()
             if next_ses is not None:
                 log.info(f"Including sub epoch summary{next_ses}")
@@ -685,8 +746,7 @@ class Timelord:
             )
 
             if next_ses is None or next_ses.new_difficulty is None:
-                self.total_unfinished += len(self.overflow_blocks)
-                self.unfinished_blocks = self.overflow_blocks
+                self.unfinished_blocks = self.overflow_blocks.copy()
             else:
                 # No overflow blocks in a new epoch
                 self.unfinished_blocks = []
@@ -696,22 +756,34 @@ class Timelord:
             await self._handle_subslot_end()
 
     async def _handle_failures(self):
-        while len(self.vdf_failures) > 0:
-            log.error(f"Vdf clients failed {self.vdf_failures_count} times.")
-            failed_chain = self.vdf_failures[0]
-            if failed_chain in self.allows_iters:
-                self.allows_iters.remove(failed_chain)
-            if failed_chain not in self.unspawned_chains:
-                self.unspawned_chains.append(failed_chain)
-            if failed_chain in self.chain_type_to_stream:
-                del self.chain_type_to_stream[failed_chain]
-            ip_iters = self.last_state.get_last_ip()
-            sub_slot_iters = self.last_state.get_sub_slot_iters()
-            left_subslot_iters = sub_slot_iters - ip_iters
-            self.iters_to_submit[failed_chain] = []
-            self.iters_to_submit[failed_chain].append(left_subslot_iters)
-            self.iters_submitted[failed_chain] = []
-            self.vdf_failures = self.vdf_failures[1:]
+        if len(self.vdf_failures) > 0:
+            # This can happen if one of the VDF processes has an issue. In this case, we abort all other
+            # infusion points and signage points, and go straight to the end of slot, so we avoid potential
+            # issues with the number of iterations that failed.
+
+            failed_chain, proof_label = self.vdf_failures[0]
+            log.error(
+                f"Vdf clients failed {self.vdf_failures_count} times. Last failure: {failed_chain}, "
+                f"label {proof_label}, current: {self.num_resets}"
+            )
+            if proof_label == self.num_resets:
+                await self._reset_chains(only_eos=True)
+            self.vdf_failure_time = time.time()
+            self.vdf_failures = []
+
+        # If something goes wrong in the VDF client due to a failed thread, we might get stuck in a situation where we
+        # are waiting for that client to finish. Usually other peers will finish the VDFs and reset us. In the case that
+        # there are no other timelords, this reset should bring the timelord back to a running state.
+        if time.time() - self.vdf_failure_time < self.constants.SUB_SLOT_TIME_TARGET * 3:
+            # If we have recently had a failure, allow some more time to finish the slot (we can be up to 3x slower)
+            active_time_threshold = self.constants.SUB_SLOT_TIME_TARGET * 3
+        else:
+            # If there were no failures recently trigger a reset after 60 seconds of no activity.
+            # Signage points should be every 9 seconds
+            active_time_threshold = 60
+        if time.time() - self.last_active_time > active_time_threshold:
+            log.error(f"Not active for {active_time_threshold} seconds, restarting all chains")
+            await self._reset_chains()
 
     async def _manage_chains(self):
         async with self.lock:
@@ -725,20 +797,27 @@ class Timelord:
                     # We've got a new peak, process it.
                     if self.new_peak is not None:
                         await self._handle_new_peak()
-                    # A subslot ended, process it.
-                    if self.new_subslot_end is not None:
-                        await self._handle_subslot_end()
                 # Map free vdf_clients to unspawned chains.
                 await self._map_chains_with_vdf_clients()
                 async with self.lock:
                     # Submit pending iterations.
                     await self._submit_iterations()
-                    # Check for new signage point and broadcast it if present.
-                    await self._check_for_new_sp()
+
+                    not_finished_iters = [
+                        it for it in self.iters_submitted[Chain.REWARD_CHAIN] if it not in self.iters_finished
+                    ]
+                    if len(not_finished_iters) == 0:
+                        await asyncio.sleep(0.1)
+                        continue
+                    selected_iter = min(not_finished_iters)
+
                     # Check for new infusion point and broadcast it if present.
-                    await self._check_for_new_ip()
+                    await self._check_for_new_ip(selected_iter)
+                    # Check for new signage point and broadcast it if present.
+                    await self._check_for_new_sp(selected_iter)
                     # Check for end of subslot, respawn chains and build EndOfSubslotBundle.
-                    await self._check_for_end_of_subslot()
+                    await self._check_for_end_of_subslot(selected_iter)
+
             except Exception:
                 tb = traceback.format_exc()
                 log.error(f"Error while handling message: {tb}")
@@ -756,6 +835,8 @@ class Timelord:
         header_hash: Optional[bytes32] = None,
         height: Optional[uint32] = None,
         field_vdf: Optional[uint8] = None,
+        # Labels a proof to the current state only
+        proof_label: Optional[int] = None,
     ):
         disc: int = create_discriminant(challenge, self.constants.DISCRIMINANT_SIZE_BITS)
 
@@ -792,14 +873,14 @@ class Timelord:
             except (asyncio.IncompleteReadError, ConnectionResetError, Exception) as e:
                 log.warning(f"{type(e)} {e}")
                 async with self.lock:
-                    self.vdf_failures.append(chain)
+                    self.vdf_failures.append((chain, proof_label))
                     self.vdf_failures_count += 1
                 return
 
             if ok.decode() != "OK":
                 return
 
-            log.info("Got handshake with VDF client.")
+            log.debug("Got handshake with VDF client.")
             if not self.sanitizer_mode:
                 async with self.lock:
                     self.allows_iters.append(chain)
@@ -825,7 +906,7 @@ class Timelord:
                 ) as e:
                     log.warning(f"{type(e)} {e}")
                     async with self.lock:
-                        self.vdf_failures.append(chain)
+                        self.vdf_failures.append((chain, proof_label))
                         self.vdf_failures_count += 1
                     break
 
@@ -835,7 +916,7 @@ class Timelord:
                 except Exception:
                     pass
                 if msg == "STOP":
-                    log.info(f"Stopped client running on ip {ip}.")
+                    log.debug(f"Stopped client running on ip {ip}.")
                     async with self.lock:
                         writer.write(b"ACK")
                         await writer.drain()
@@ -853,7 +934,7 @@ class Timelord:
                     ) as e:
                         log.warning(f"{type(e)} {e}")
                         async with self.lock:
-                            self.vdf_failures.append(chain)
+                            self.vdf_failures.append((chain, proof_label))
                             self.vdf_failures_count += 1
                         break
 
@@ -893,7 +974,8 @@ class Timelord:
                         log.error("Invalid proof of time!")
                     if not self.sanitizer_mode:
                         async with self.lock:
-                            self.proofs_finished.append((chain, vdf_info, vdf_proof))
+                            assert proof_label is not None
+                            self.proofs_finished.append((chain, vdf_info, vdf_proof, proof_label))
                     else:
                         async with self.lock:
                             writer.write(b"010")
@@ -905,10 +987,10 @@ class Timelord:
                             vdf_info, vdf_proof, header_hash, height, field_vdf
                         )
                         if self.server is not None:
-                            message = make_msg(ProtocolMessageTypes.respond_compact_vdf_timelord, response)
+                            message = make_msg(ProtocolMessageTypes.respond_compact_proof_of_time, response)
                             await self.server.send_to_all([message], NodeType.FULL_NODE)
         except ConnectionResetError as e:
-            log.info(f"Connection reset with VDF client {e}")
+            log.debug(f"Connection reset with VDF client {e}")
 
     async def _manage_discriminant_queue_sanitizer(self):
         while not self._shut_down:
