@@ -2,12 +2,10 @@ from dataclasses import dataclass
 from typing import List
 
 from chia.types.blockchain_format.coin import Coin
-from chia.types.blockchain_format.program import Program
-from chia.util.chain_utils import additions_for_solution, announcements_for_solution
+from chia.types.blockchain_format.program import Program, INFINITE_COST
+from chia.util.chain_utils import additions_for_solution
 from chia.util.streamable import Streamable, streamable
 
-from .announcement import Announcement
-
 
 @dataclass(frozen=True)
 @streamable
@@ -23,7 +21,4 @@ class CoinSolution(Streamable):
     solution: Program
 
     def additions(self) -> List[Coin]:
-        return additions_for_solution(self.coin.name(), self.puzzle_reveal, self.solution)
-
-    def announcements(self) -> List[Announcement]:
-        return announcements_for_solution(self.coin.name(), self.puzzle_reveal, self.solution)
+        return additions_for_solution(self.coin.name(), self.puzzle_reveal, self.solution, INFINITE_COST)
