@@ -5,7 +5,7 @@ import socket
 import time
 import traceback
 from pathlib import Path
-from typing import Callable, Dict, List, Optional, Set, Tuple, Union
+from typing import Callable, Dict, List, Optional, Set, Tuple, Union, Any
 
 from blspy import PrivateKey
 
@@ -553,6 +553,33 @@ class WalletNode:
             fork_height = None
             if peak is not None:
                 fork_height = self.wallet_state_manager.sync_store.get_potential_fork_point(peak.header_hash)
+                our_peak_height = self.wallet_state_manager.blockchain.get_peak_height()
+                ses_heigths = self.wallet_state_manager.blockchain.get_ses_heights()
+                if len(ses_heigths) > 2 and our_peak_height is not None:
+                    ses_heigths.sort()
+                    max_fork_ses_height = ses_heigths[-3]
+                    # This is fork point in SES in case where fork was not detected
+                    if (
+                        self.wallet_state_manager.blockchain.get_peak_height() is not None
+                        and fork_height == max_fork_ses_height
+                    ):
+                        peers = self.server.get_full_node_connections()
+                        for peer in peers:
+                            # Grab a block at peak + 1 and check if fork point is actually our current height
+                            potential_height = uint32(our_peak_height + 1)
+                            block_response: Optional[Any] = await peer.request_header_blocks(
+                                wallet_protocol.RequestHeaderBlocks(potential_height, potential_height)
+                            )
+                            if block_response is not None and isinstance(
+                                block_response, wallet_protocol.RespondHeaderBlocks
+                            ):
+                                our_peak = self.wallet_state_manager.blockchain.get_peak()
+                                if (
+                                    our_peak is not None
+                                    and block_response.header_blocks[0].prev_header_hash == our_peak.header_hash
+                                ):
+                                    fork_height = our_peak_height
+                                break
             if fork_height is None:
                 fork_height = uint32(0)
             await self.wallet_state_manager.blockchain.warmup(fork_height)
