@@ -13,13 +13,12 @@ from chia.full_node.coin_store import CoinStore
 from chia.types.blockchain_format.coin import Coin
 from chia.types.coin_record import CoinRecord
 from chia.types.full_block import FullBlock
-from chia.util.db_wrapper import DBWrapper
+from chia.util.generator_tools import run_and_get_removals_and_additions
 from chia.util.ints import uint64
 from chia.util.wallet_tools import WalletTool
+from chia.util.db_wrapper import DBWrapper
 from tests.setup_nodes import bt, test_constants
 
-WALLET_A = WalletTool()
-
 
 @pytest.fixture(scope="module")
 def event_loop():
@@ -29,6 +28,8 @@ def event_loop():
 
 constants = test_constants
 
+WALLET_A = WalletTool(constants)
+
 
 def get_future_reward_coins(block: FullBlock) -> Tuple[Coin, Coin]:
     pool_amount = calculate_pool_reward(block.height)
@@ -94,11 +95,11 @@ class TestCoinStore:
             should_be_included.add(farmer_coin)
             should_be_included.add(pool_coin)
             if block.is_transaction_block():
-                removals, additions = block.tx_removals_and_additions()
+                removals, additions = run_and_get_removals_and_additions(block, constants.MAX_BLOCK_COST_CLVM)
 
                 assert block.get_included_reward_coins() == should_be_included_prev
 
-                await coin_store.new_block(block)
+                await coin_store.new_block(block, additions, removals)
 
                 for expected_coin in should_be_included_prev:
                     # Check that the coinbase rewards are added
@@ -136,7 +137,8 @@ class TestCoinStore:
         # Save/get block
         for block in blocks:
             if block.is_transaction_block():
-                await coin_store.new_block(block)
+                removals, additions = run_and_get_removals_and_additions(block, constants.MAX_BLOCK_COST_CLVM)
+                await coin_store.new_block(block, additions, removals)
                 coins = block.get_included_reward_coins()
                 records = [await coin_store.get_coin_record(coin.name()) for coin in coins]
 
@@ -164,7 +166,8 @@ class TestCoinStore:
 
         for block in blocks:
             if block.is_transaction_block():
-                await coin_store.new_block(block)
+                removals, additions = run_and_get_removals_and_additions(block, constants.MAX_BLOCK_COST_CLVM)
+                await coin_store.new_block(block, additions, removals)
                 coins = block.get_included_reward_coins()
                 records: List[Optional[CoinRecord]] = [await coin_store.get_coin_record(coin.name()) for coin in coins]
 
