@@ -2,10 +2,12 @@ import unittest
 from dataclasses import dataclass
 from typing import List, Optional
 
+from clvm_tools import binutils
 from pytest import raises
 
 from chia.protocols.wallet_protocol import RespondRemovals
 from chia.types.blockchain_format.coin import Coin
+from chia.types.blockchain_format.program import Program
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.full_block import FullBlock
 from chia.types.weight_proof import SubEpochChallengeSegment
@@ -161,6 +163,27 @@ class TestStreamable(unittest.TestCase):
         TestClassBool.from_bytes(bytes([0]))
         TestClassBool.from_bytes(bytes([1]))
 
+    def test_ambiguous_deserialization_program(self):
+        @dataclass(frozen=True)
+        @streamable
+        class TestClassProgram(Streamable):
+            a: Program
+
+        program = Program.to(binutils.assemble("()"))
+
+        TestClassProgram.from_bytes(bytes(program))
+
+        with raises(AssertionError):
+            TestClassProgram.from_bytes(bytes(program) + b"9")
+
+    def test_streamable_empty(self):
+        @dataclass(frozen=True)
+        @streamable
+        class A(Streamable):
+            pass
+
+        assert A.from_bytes(bytes(A())) == A()
+
 
 if __name__ == "__main__":
     unittest.main()
